﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Media;
using Buddy.Coroutines;
using ff14bot;
using ff14bot.AClasses;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.RemoteWindows;
using Newtonsoft.Json;
using TreeSharp;
using System.ComponentModel;
using ff14bot.Navigation;
using Clio.Utilities;
using System.Diagnostics;
using System.Threading;

namespace GrandCompanySealsSpenderPlugin
{
    // ReSharper disable once UnusedMember.Global
    public class GrandCompanySealsSpenderPlugin : BotPlugin
    {
        #region Properties

        private static Stopwatch timerToPreventRepurchase = new Stopwatch();

        private static bool SealsFull
        {
            get
            {

                //Logging.Write(Colors.Aqua, "[GrandCompanySealsSpenderPlugin] Checking current amount of Seals.");
                if (!timerToPreventRepurchase.IsRunning || timerToPreventRepurchase.ElapsedMilliseconds > (Settings.Instance.VendorDelay * 1000 * 60))
                {
                    return InventoryManager.GetBagByInventoryBagId(InventoryBagId.Currency)[3].Count > Settings.Instance.SealsExceed;
                }
                return false;
            }
        }

        #endregion

        #region Buy GC Item Logic

        public static async Task<bool> SpendGCSealsButtonPressed()
        {

            Vector3 vendorLocation1 = new Vector3(-141.9456f, 4.1f, -107.5568f);
            if (Core.Me.Distance(vendorLocation1) < 3) BuyItemsNow();
            else Logging.Write(Colors.Aqua, "[GrandCompanySealsSpenderPlugin] Please manually move to Vendor in order to begin purchase.");
            return false;
        }

        private static async Task<bool> SpendGCSeals()
        {
            if (Settings.Instance.ItemName == "None" || Settings.Instance.ItemName == null || Settings.Instance.ItemNameIndex == 0)
            {
                Logging.Write(Colors.Aqua, "[GrandCompanySealsSpenderPlugin] Please select a new Grand Company Item from the settings window to spend seals on.");
                return false;
            }

            if (FishingManager.State != FishingState.None)
            {
                Actionmanager.DoAction(299, Core.Me);
                await Coroutine.Wait(5000, () => FishingManager.State == FishingState.None);
            }

            if (GatheringManager.WindowOpen)
            {
                Logging.Write(Colors.Aqua, "[GrandCompanySealsSpender] Currently Gathering Items, waiting until we are done.");
                return false;
            }

            if (CraftingManager.IsCrafting)
            {
                Logging.Write(Colors.Pink, "[GrandCompanySealsSpender] Currently Crafting Items, waiting until we are done.");
                return false;
            }

            if (CraftingLog.IsOpen)
            {
                Logging.Write(Colors.Pink, "[GrandCompanySealsSpender] Closing Crafting Log so we can spend seals.");
                CraftingLog.Close();
            }

            if (Core.Me.IsMounted)
            {
                Logging.Write(Colors.Aqua, "[GrandCompanySealsSpender] Dismounting so we can drink medicine.");
                await CommonTasks.StopAndDismount();
            }

            Logging.Write(Colors.Aqua, "[GrandCompanySealsSpender] Now moving to Seals vendor");
            await BuyItem();

            return true;
        }

        private static async Task<bool> BuyItem()
        {
            if (Core.Me.InCombat)
            {
                Logging.Write(Colors.Aqua, "[GrandCompanySealsSpender] In combat; waiting until it is resolved to retry");
                return false;
            }

            if (WorldManager.ZoneId != WorldManager.GetZoneForAetheryteId(9))
            {
                if (!WorldManager.HasAetheryteId(9))
                {
                    Logging.Write(Colors.SandyBrown, "[GrandCompanySealsSpender] Missing aetheryte to GC vendor");
                    return false;
                }
                Poi.Clear("We're teleporting to another zone.");
                Navigator.Stop();
                WorldManager.TeleportById(9);
                await Coroutine.Wait(15000, () => (WorldManager.ZoneId == WorldManager.GetZoneForAetheryteId(9)) || Core.Me.InCombat);
                if (Core.Me.InCombat)
                {
                    Logging.Write(Colors.Aqua, "[GrandCompanySealsSpender] Attacked while teleporting");
                    return false;
                }
                else if (WorldManager.ZoneId != WorldManager.GetZoneForAetheryteId(9))
                {
                    Logging.Write(Colors.Aqua, "[GrandCompanySealsSpender] Error teleporting to GC vendor");
                    return false;
                }
            }

            timerToPreventRepurchase.Restart();

            Vector3 vendorLocation1 = new Vector3(-141.9456f, 4.1f, -107.5568f);
            //Vector3 vendorLocation2 = new Vector3(-143.4501f, 4.549927f, -108.5374f);
            while (Core.Me.Distance(vendorLocation1) >= 3)
            {
                Navigator.MoveTo(vendorLocation1);
                await Coroutine.Yield();
            }

            await BuyItemsNow();

            return true;
        }


        private static async Task<bool> BuyItemsNow()
        {
            //Debugger.Break();
            Logging.Write(Colors.Aqua, "[GrandCompanySealsSpender] Now purchasing vendor item " + Settings.Instance.ItemName);

            uint vendorId = 1002390;
            Stopwatch timer = new Stopwatch();
            timer.Start();
            while (!GrandCompanyExchangeWindow.IsOpen && timer.ElapsedMilliseconds < 5000)
            {
                GameObjectManager.GetObjectByNPCId(vendorId).Interact();
                //await Coroutine.Sleep(1000);
                RaptureAtkUnitManager.Update();
                if (Coroutine.Current != null)
                {
                    await Coroutine.Wait(50, () => GrandCompanyExchangeWindow.IsOpen);
                    RaptureAtkUnitManager.Update();
                }
                else
                {
                        //Debugger.Break();
                        Thread.Sleep(50);
                        Thread.Yield();
                        //Debugger.Break();
                        RaptureAtkUnitManager.Update();
                }
            }
            RaptureAtkUnitManager.Update();
            if (!GrandCompanyExchangeWindow.IsOpen)
            {
                Logging.Write(Colors.Aqua, "[GrandCompanySealsSpender] Error interacting with GC vendor");
                return false;
            }

            GrandCompanyExchangeWindow.MoveToTab();
            //await Coroutine.Sleep(1000);
            GrandCompanyExchangeWindow.MoveToType();
            while (InventoryManager.GetBagByInventoryBagId(InventoryBagId.Currency)[3].Count >= Settings.Instance.SealsBelow)
            {
                timer.Restart();
                while (!GCESelectyesno.IsOpen && timer.ElapsedMilliseconds < 5000)
                {
                    GrandCompanyExchangeWindow.BuyItem();
                    //await Coroutine.Sleep(1000);
                    RaptureAtkUnitManager.Update();
                    if (Coroutine.Current != null)
                    {
                        await Coroutine.Wait(50, () => GCESelectyesno.IsOpen);
                        RaptureAtkUnitManager.Update();
                    }
                    else
                    {
                        //Debugger.Break();
                        Thread.Sleep(50);
                        Thread.Yield();
                        //Debugger.Break();
                        RaptureAtkUnitManager.Update();
                    }
                }
                RaptureAtkUnitManager.Update();

                if (!GCESelectyesno.IsOpen)
                {
                    Logging.Write(Colors.Aqua, "[GrandCompanySealsSpender] Error purchasing from GC vendor: might have run out of Seals");
                    break;
                }
                else
                {
                    timer.Restart();
                    while (GCESelectyesno.IsOpen && timer.ElapsedMilliseconds < 5000)
                    {
                        GCESelectyesno.ClickOk();
                        //await Coroutine.Sleep(1000);
                        RaptureAtkUnitManager.Update();
                        if (Coroutine.Current != null)
                        {
                            await Coroutine.Wait(50, () => !GCESelectyesno.IsOpen);
                            RaptureAtkUnitManager.Update();
                        }
                        else
                        {
                            //Debugger.Break();
                            Thread.Sleep(50);
                            Thread.Yield();
                            //Debugger.Break();
                            RaptureAtkUnitManager.Update();
                        }
                    }
                    RaptureAtkUnitManager.Update();
                    if (GCESelectyesno.IsOpen)
                    {
                        Logging.Write(Colors.Aqua, "[GrandCompanySealsSpender] Error purchasing from GC vendor: unable to close purchase confirmation window");
                        if (Coroutine.Current != null)
                        {
                            Logging.Write(Colors.Aqua, "[GrandCompanySealsSpender] Stopping bot to prevent issues due to purchase confirmation window remaining open");
                            await TreeRoot.StopGently();
                            return true;
                        }
                    }
                    else
                    {
                        Logging.Write(Colors.Aqua, "[GrandCompanySealsSpender] Purchasing vendor item '" + Settings.Instance.ItemName + "' at " + System.DateTime.Now.ToString());
                    }
                }
                if (InventoryManager.GetBagByInventoryBagId(InventoryBagId.Currency)[3].Count < Settings.Instance.SealsBelow) break;
            }
            Logging.Write(Colors.Aqua, "[GrandCompanySealsSpender] Finished spending seals, closing window");
            GrandCompanyExchangeWindow.CloseWindow();
            return true;
        }

        public class GrandCompanyExchangeWindow
        {
            public static bool IsOpen
            {
                get
                {
                    return RaptureAtkUnitManager.GetWindowByName("GrandCompanyExchange") != null;
                }
            }

            public static void MoveToTab()
            {
                var window = RaptureAtkUnitManager.GetWindowByName("GrandCompanyExchange");
                if (window != null)
                {
                    window.SendAction(2, 1, 1, (uint)Settings.Instance.ItemRank, (uint)Settings.Instance.ItemRank);
                }
            }

            public static void MoveToType()
            {
                var window = RaptureAtkUnitManager.GetWindowByName("GrandCompanyExchange");
                if (window != null)
                {
                    window.SendAction(2, 2, 2, (uint)(Settings.Instance.ItemType+1), (uint)(Settings.Instance.ItemType + 1));
                }
            }

            public static void BuyItem()
            {
                var window = RaptureAtkUnitManager.GetWindowByName("GrandCompanyExchange");
                if (window != null)
                {
                    window.SendAction(2, 1, 0, 1, (uint)Settings.Instance.ItemNameIndex);
                }
            }


            public static void CloseWindow()
            {
                var window = RaptureAtkUnitManager.GetWindowByName("GrandCompanyExchange");
                if (window != null)
                {
                    window.SendAction(1, 3, 0xFFFFFFFF);
                }
            }
        }
        
        public class GCESelectyesno
        {
            public static bool IsOpen
            {
                get
                {
                    return RaptureAtkUnitManager.GetWindowByName("SelectYesno") != null;
                }
            }

            public static void ClickOk()
            {
                var window = RaptureAtkUnitManager.GetWindowByName("SelectYesno");
                if (window != null)
                {
                    window.SendAction(1, 3, 0);
                }
            }
        }

        #endregion

        #region Fields

        private Composite _coroutine;
        private Gui _settingsForm;

        #endregion

        #region Required Overrides

        public override string Author
        {
            get { return "Cloud30000"; }
        }

        public override string Name
        {
            get { return "GrandCompanySealsSpender"; }
        }

        public override Version Version
        {
            get { return new Version(1, 0, 0, 0); }
        }

        #endregion

        #region Optional Overrides

        public override void OnInitialize()
        {
            _coroutine = new Decorator(c => Core.Me.IsAlive && SealsFull && !Core.Me.InCombat, new ActionRunCoroutine(r => SpendGCSeals()));
        }

        public override void OnEnabled()
        {
            TreeRoot.OnStart += OnBotStart;
            TreeRoot.OnStop += OnBotStop;
            TreeHooks.Instance.OnHooksCleared += OnOnHooksCleared;

            if (TreeRoot.IsRunning)
            {
                AddHooks();
            }
        }

        public override void OnDisabled()
        {
            TreeRoot.OnStart -= OnBotStart;
            TreeRoot.OnStop -= OnBotStop;
            TreeHooks.Instance.OnHooksCleared -= OnOnHooksCleared;
            RemoveHooks();
        }

        public override void OnShutdown()
        {
            TreeRoot.OnStart -= OnBotStart;
            TreeRoot.OnStop -= OnBotStop;
            TreeHooks.Instance.OnHooksCleared -= OnOnHooksCleared;
            RemoveHooks();
        }

        public override bool WantButton
        {
            get { return true; }
        }

        public override void OnButtonPress()
        {
            if (_settingsForm == null || _settingsForm.IsDisposed || _settingsForm.Disposing)
                _settingsForm = new Gui();

            _settingsForm.ShowDialog();
        }

        #endregion

        #region Hook Logic

        private void AddHooks()
        {
            Logging.Write(Colors.Aqua, "Adding GrandCompanySealsSpender Hook");
            TreeHooks.Instance.InsertHook("TreeStart", 0, _coroutine);
        }

        private void RemoveHooks()
        {
            Logging.Write(Colors.Aqua, "Removing GrandCompanySealsSpender Hook");
            TreeHooks.Instance.RemoveHook("TreeStart", _coroutine);
        }

        #endregion

        #region Event Handlers

        private void OnBotStop(BotBase bot)
        {
            RemoveHooks();
        }

        private void OnBotStart(BotBase bot)
        {
            AddHooks();
        }

        private void OnOnHooksCleared(object sender, EventArgs e)
        {
            AddHooks();
        }

        #endregion
    }

    #region Settings Class

    public class Settings : JsonSettings
    {
        [JsonIgnore]
        private static Settings _instance;
        // ReSharper disable once UnusedParameter.Local
        // ReSharper disable once MemberCanBePrivate.Global
        public Settings(string filename)
            : base(Path.Combine(CharacterSettingsDirectory, "GrandCompanySealsSpender.json"))
        {
        }

        public static Settings Instance
        {
            get { return _instance ?? (_instance = new Settings("Settings")); }
        }

        [Setting]
        public string ItemName { get; set; }
        [Setting]
        [DefaultValue(0)]
        public int ItemRank { get; set; }
        [Setting]
        [DefaultValue(0)]
        public int ItemType { get; set; }
        [Setting]
        [DefaultValue(0)]
        public int ItemNameIndex { get; set; }
        [Setting]
        [DefaultValue(49999)]
        public int SealsExceed { get; set; }
        [Setting]
        [DefaultValue(1)]
        public int SealsBelow { get; set; }
        [Setting]
        [DefaultValue(30)]
        public int VendorDelay { get; set; }
    }

    #endregion
}